==> gcma-0.1.24.tar.gz: gcma-0.1.24/gcma/io/baseio.py <==
from __future__ import annotations

import inspect
import logging
from copy import deepcopy
from typing import Union, TypeVar, Set, List, Tuple

import numpy as np
import treefiles as tf

TValue = Union[int, float, np.ndarray]
TRegistered = Set[str]


class BaseIO:
    def __init__(self, name: str, value: TValue = None, pretty_name: str = None):
        self.name = name
        self.pretty_name = pretty_name
        self.value = value

        self.registered: TRegistered = {"name", "pretty_name", "value"}

    @property
    def _class_name(self) -> str:
        return type(self).__name__

    @property
    def dict(self) -> dict:
        return self.to_dict()

    def get_pname(self) -> str:
        return self.name if self.pretty_name is None else self.pretty_name

    def __repr__(self):
        return f"<{self._class_name}.{self.name}>"

    def to_dict(self) -> dict:
        return {k: getattr(self, k) for k in self.registered}

    def copy(self):
        return deepcopy(self)

    @classmethod
    def from_dict(cls, d: dict) -> T:
        d, s = dict(d), dict()

        keys = []
        for base_class in reversed(cls.mro()):
            sig = inspect.signature(base_class.__init__)
            keys.extend(sig.parameters.keys())
        keys = [k for k in keys if k not in ["self", "args", "kwargs"]]

        for k in keys:
            if k not in d:
                msg = f"Cannot cast dict to <{cls.__name__}>: {k!r} not present"
                log.critical(msg)
                raise TypeError(msg)

        s = {k: d.pop(k) for k in keys}

        c = cls(**s)
        for k in c.registered:
            if not hasattr(c, k):
                setattr(c, k, d.get(k))
        return c


class IOEncoder(tf.NumpyEncoder):
    def default(self, obj):
        if isinstance(obj, BaseIO):
            return obj.to_dict()
        return super().default(obj)


class Bases(dict):
    inner_class = None

    def __init__(self, *items: Union[T, List[T], Bases, dict]):
        ln = len(items)
        if ln > 0:
            if ln == 1:
                x = items[0]
                if isinstance(x, BaseIO):
                    items = [x]
                elif isinstance(x, dict):
                    y = list(x.values())[0]
                    if isinstance(y, BaseIO):
                        items = list(x.values())
                    elif isinstance(y, dict):
                        items = [self.inner_class.from_dict(z) for z in x.values()]
                elif len(x) > 0:
                    if isinstance(x[0], BaseIO):
                        items = x
        super().__init__({k.name: k for k in items})

    @property
    def elems(self) -> List[T]:
        return list(self.values())

    def copy(self) -> TBases:
        return type(self)({k: v.copy() for k, v in self.items()})

    @property
    def table(self) -> tf.Table:
        h, d = self.build_table()
        return tf.Table(header=h, rows=d)

    THeader = List[str]
    TData = List[List[TValue]]

    def build_table(self) -> Tuple[THeader, TData]:
        raise NotImplementedError

    def assign(self, values: Union[np.ndarray, List[TValue]], att: str = "value"):
        cop = self.copy()
        for x, v in zip(cop.values(), values):
            setattr(x, att, v)
        return cop

    @property
    def vals(self):
        return [x.value for x in self.values()]

    def print(self):
        return f"{type(self).__name__}:\n{self.table()}"


T = TypeVar("T", bound=BaseIO)
TBases = TypeVar("TBases", bound=Bases)


log = logging.getLogger(__name__)

==> gcma-0.1.25.tar.gz: gcma-0.1.25/gcma/io/baseio.py <==
from __future__ import annotations

import inspect
import json
import logging
from copy import deepcopy
from typing import Union, TypeVar, Set, List, Tuple

import numpy as np
import treefiles as tf

TValue = Union[int, float, np.ndarray]
TRegistered = Set[str]


class BaseIO:
    def __init__(
        self, name: str, value: TValue = None, pretty_name: str = None, unit: str = None
    ):
        self.name = name
        self.pretty_name = pretty_name
        self.value = value
        self.unit = unit

        self.registered: TRegistered = {"name", "pretty_name", "value", "unit"}

    @property
    def _class_name(self) -> str:
        return type(self).__name__

    @property
    def dict(self) -> dict:
        return self.to_dict()

    def get_pname(self) -> str:
        return self.name if self.pretty_name is None else self.pretty_name

    def __repr__(self):
        return f"<{self._class_name}.{self.name}>"

    def to_dict(self) -> dict:
        return {k: getattr(self, k) for k in self.registered}

    def copy(self):
        return deepcopy(self)

    @classmethod
    def from_dict(cls, d: dict) -> T:
        d, s = dict(d), dict()

        keys = []
        for base_class in reversed(cls.mro()):
            sig = inspect.signature(base_class.__init__)
            keys.extend(sig.parameters.keys())
        keys = [k for k in keys if k not in ["self", "args", "kwargs"]]

        for k in keys:
            if k not in d:
                msg = f"Cannot cast dict to <{cls.__name__}>: {k!r} not present"
                log.critical(msg)
                raise TypeError(msg)

        s = {k: d.pop(k) for k in keys}

        c = cls(**s)
        for k in c.registered:
            if not hasattr(c, k):
                setattr(c, k, d.get(k))
        return c


class IOEncoder(tf.NumpyEncoder):
    def default(self, obj):
        if isinstance(obj, BaseIO):
            return obj.to_dict()
        return super().default(obj)


class Bases(dict):
    inner_class = None

    def __init__(self, *items: Union[T, List[T], Bases, dict]):
        ln = len(items)
        if ln > 0:
            if ln == 1:
                x = items[0]
                if isinstance(x, BaseIO):
                    items = [x]
                elif isinstance(x, dict):
                    y = list(x.values())[0]
                    if isinstance(y, BaseIO):
                        items = list(x.values())
                    elif isinstance(y, dict):
                        items = [self.inner_class.from_dict(z) for z in x.values()]
                elif len(x) > 0:
                    if isinstance(x[0], BaseIO):
                        items = x
        super().__init__({k.name: k for k in items})

    @property
    def elems(self) -> List[T]:
        return list(self.values())

    def copy(self) -> TBases:
        return type(self)({k: v.copy() for k, v in self.items()})

    @property
    def table(self) -> tf.Table:
        h, d = self.build_table()
        return tf.Table(header=h, rows=d)

    THeader = List[str]
    TData = List[List[TValue]]

    def build_table(self) -> Tuple[THeader, TData]:
        raise NotImplementedError

    def assign(self, values: Union[np.ndarray, List[TValue]], att: str = "value"):
        cop = self.copy()
        for x, v in zip(cop.values(), values):
            setattr(x, att, v)
        return cop

    @property
    def vals(self):
        return [x.value for x in self.values()]

    def print(self):
        return f"{type(self).__name__}:\n{self.table()}"

    def to_dict(self):
        return {k: v.to_dict() for k, v in self.items()}


T = TypeVar("T", bound=BaseIO)
TBases = TypeVar("TBases", bound=Bases)


log = logging.getLogger(__name__)
